# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de usuario. Este controlador es utilizado desde las secciones:
# * Sección administracion: crear, modificar, eliminar usuarios
# * Sección entrada: autentifica y carga en la sessión
# * En cualquier sección: salir

class UsuarioController < ApplicationController

  include SimpleCaptcha::ControllerHelpers

  before_filter :usuario_desde_parametros, :only => [:listado, :editar_nuevo, :modificar_crear, :eliminar, :listado_relaciones,
                                                     :asignar_relacion, :modificar_crear_relacion, :desasignar_relacion ]

  ############################################
  ## Acciones de usuario
  ############################################

  # Entrada al sistema. Valida si debe usarse la identificacio propia (render de identificacion) o externa 
  def entrada
    # Averiguamos si ya tenemos sesion en cuyo caso redirigimos y esperamos que los filtros la validen
    if session && session[:usuario_identificado_id]
      redirect_to params[:ir_a] || session[:ir_a] || {:seccion => :inicio, :controller => "/info"}
    # Si no tenemos, enviamos a la autentificacion propia o de plugin
    else
      # Comprueba si debe hacerse una auth externa
      auth_externa = Rails.configuration.external_auth.first
      if auth_externa && auth_externa.respond_to?(:auth_endpoint) && auth_externa.auth_endpoint
        redirect_to auth_externa.auth_endpoint || :identifica
      else
        render :identificacion
      end
    end
  end
  
  # Presenta el formulario de identificacion y hace la validacion
  def identificacion
    if params[:usuario] && (usuario = Usuario.identificacion(params[:usuario][:nombre], params[:usuario][:contrasena]))
      redirect_to sesion_iniciar(usuario)
    elsif params[:usuario]
      msg_error _("Identificación incorrecta.<br>Compruebe su usuario y contraseña.")
      # Hacemos este redirect to back para ocultar el metodo cuando entramos de cero
      redirect_to :back
    end
  end

  # en cualquier sección: pon a nil el usuario_identificado en la sessión
  def salir
    sesion_terminar
    # Comprueba si estamos usando una auth externa
    auth_externa = Rails.configuration.external_auth.first
    # Y redirige a donde deba
    redirect_to (auth_externa && auth_externa.respond_to?(:logout_url) && auth_externa.logout_url) ? auth_externa.logout_url : entrada_path
  end

  # Presenta la modal de datos personales
  def datos_personales
    if request.xhr?
      @usuario = @usuario_identificado
      render :partial => "datos_personales"
    else
      redirect_to :proyectos
    end
  end

  # Modificacion de datos personales
  def cambiar_datos_personales
    @usuario = @usuario_identificado
    if Usuario.hash_contrasena(params[:comprobar][:contrasena_antigua]) == @usuario.contrasena and params[:comprobar][:contrasena_nueva] == params[:comprobar][:contrasena_repetida]
      @usuario.update_attribute :contrasena,  Usuario.hash_contrasena(params[:comprobar][:contrasena_nueva]) unless params[:comprobar][:contrasena_nueva] == ""
    else
      fallo_comprobacion = params[:comprobar][:contrasena_nueva] != ""
    end
    @usuario.update_attributes params[:usuario]
    # Actualiza las notificaciones de proyectos
    unless !@usuario.errors.empty? || fallo_comprobacion
      params[:notificar_proyecto].each do |notificar_proyecto|
        # Como podemos tener asignado el proyecto directamente o a traves de un grupo, recorremos todos...
        UsuarioXProyecto.all(:conditions => {:usuario_id => @usuario.id, :proyecto_id => notificar_proyecto[0]}).each do |uxp|
          uxp.update_attribute :notificar, (notificar_proyecto[1] == "1")
        end
      end if params[:notificar_proyecto]
    end
    render :update do |page|
      @mensaje = fallo_comprobacion ? _("Error en los datos de comprobación") : @usuario
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@mensaje) %><br>'
      page.call("Modalbox.resizeToContent")
    end
  end



  ############################################
  ## Métodos de gestion de usuarios
  ############################################

  # en administracion: se redirecciona por defecto a ordenado
  def index
    redirect_to :action => 'ordenado'
  end

  # en administracion: lista
  def listado
    session[:usuario_orden] ||= "nombre"
    session[:usuario_asc_desc] ||= "ASC"
    @usuario = Usuario.find_by_id params[:id]
    @usuarios = @paginado =  Usuario.paginate( :order => session[:usuario_orden] + " " + session[:usuario_asc_desc], :page => params[:page],:per_page => (session[:por_pagina]))
  end

  # en administracion: establece los parametros de ordenación 
  def ordenado
    session[:usuario_orden] = params[:orden] ? params[:orden] : "nombre" 
    session[:usuario_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC"
    redirect_to :action => "listado"
  end
  
  # en administracion: prepara el formulario de edición o creación
  def editar_nuevo
    @usuario = Usuario.find_by_id params[:id]
    render :partial => "formulario"
  end


  # en administracion: modifica o crea
  def modificar_crear
    @usuario = Usuario.find_by_id params[:id]
    @usuario ||= Usuario.new
    @usuario.update_attributes params[:usuario]
    @usuario.update_attribute :contrasena,  Usuario.hash_contrasena(params[:usuario][:contrasena]) if params[:usuario][:contrasena] && @usuario.errors.empty?
    @usuario_identificado.reload if @usuario.id == @usuario_identificado.id
    msg @usuario
    redirect_to :action => "listado"
  end

  # en administracion: añade un campo "contraseña al formulario
  def contrasena
    render :inline => "<%= contrasena 'Contraseña', 'usuario', 'contrasena', '1' %>"
  end

  # en administracion: elimina usuario.
  def eliminar
    @usuario = Usuario.find_by_id params[:id]
    @usuario.destroy
    msg_eliminar @usuario
    redirect_to :action => 'listado'
  end

  #++
  # Asigna el usuario actual a otros objetos
  #--

  # Listado de relaciones
  def listado_relaciones
    proyectos = usuario_x_proyecto
    agentes = usuario_x_agente
    libros = usuario_x_libro
    render :update do |page|
      page.replace_html params[:update], :partial => "listado_relaciones", :locals => { proyectos: proyectos, agentes: agentes, libros: libros }
    end
  end

  def asignar_relacion
    @elemento = eval("UsuarioX" + params[:tipo].capitalize).find_by_id(params[:elemento_id]) || eval("UsuarioX" + params[:tipo].capitalize).new
    # Obtenemos todos los elementos no vinculados aun
    elementos_posibles
    render(:update){ |page| page.formulario :partial => "formulario_relacion", :update => params[:update] }
  end

  def modificar_crear_relacion
    @elemento = eval("UsuarioX" + params[:tipo].capitalize).find_by_id(params[:elemento_id]) || eval("UsuarioX" + params[:tipo].capitalize).new(usuario_id: @usuario.id)
    @elemento.update_attributes(params[:elemento])
    elementos = eval("usuario_x_" + params[:tipo])
    if @elemento.errors.empty?
      render :update do |page|
        page.replace_html params[:update_listado], :partial => "listado_relaciones_elemento", :locals => { elementos: elementos, tipo: params[:tipo] }
      end
    else
      # Obtenemos todos los elementos no vinculados aun
      elementos_posibles
      render(:update) { |page| page.recargar_formulario :partial => "formulario_relacion", :mensaje => {:errors => @elemento.errors} }
    end
  end

  def desasignar_relacion
    elemento = eval("UsuarioX" + params[:tipo].capitalize).find_by_id(params[:elemento_id])
    elemento.destroy
    render (:update) {|page| page.eliminar :update => params[:update], :mensaje => {:errors => elemento.errors, :eliminar => true}}
  end
 
  #++
  # Asignar USUARIOS (ayudas para asignacion de usuarios en otros controladores)
  #--
	# en administracion: muestra el listado de usuarios y grupos vinculados
  def usuarios
    @objeto = eval( params[:objeto].split('_').map(&:capitalize).join ).find( params[:id] )
    @usuarios_x = @objeto.usuario_x_vinculado
    # En los listados de asignaciones incluiremos una caja de grupos si existe modelo que lo gestione
    @grupos_x = eval("@objeto.grupo_usuario_x_" + params[:objeto] ) if @objeto.respond_to?("grupo_usuario_x_" + params[:objeto]) 
    render :update do |page|
      page.replace_html params[:update], :partial => "usuario/usuarios"
      page.insert_html :after, params[:update] + "_usuarios", :partial => "grupo_usuario/grupos" if @grupos_x
    end 
  end

  def asignar_usuario
    @objeto = eval( params[:objeto].split('_').map(&:capitalize).join ).find( params[:id].to_s )
    @listado_usuarios = Usuario.find(:all, :order => "nombre").collect{ |u| [u.nombre, u.id] } - @objeto.usuario_x_vinculado.collect{ |u| [(u.usuario ? u.usuario.nombre : ""), u.usuario_id] }
    @usuario_actual = Usuario.find( params[:usuario_id] ) if params[:usuario_id]
    @objeto_nombre = case params[:objeto]
                       when "libro" then "Cuenta"
                       else  params[:objeto].titleize
                     end
    render :partial => "usuario/asignar_usuario", :locals => {:objeto => params[:objeto], :update => params[:update]}
  end

  def crear_modificar_asignacion
    @message = nil 
    @objeto = eval( params[:objeto].split('_').map(&:capitalize).join ).find( params[:id].to_s )
    if params[:usuario_id]
      eval( "@usuario_relacionado = UsuarioX" + params[:objeto].split('_').map(&:capitalize).join + ".find( :first, :conditions => {:usuario_id => params[:usuario_id], :" + params[:objeto] + "_id =>  params[:id]} )" )
      if params[:usuario] && params[:usuario][:rol]
        params[:usuario][:notificar] = (params[:usuario][:rol] != "auditor" && params[:usuario][:rol] != "configurador")
        @usuario_relacionado.update_attributes params[:usuario]
      end
    # Cuando no hay params[:usuario_id]? -> Cuando se crea nuevo
    else
      @usuario_relacionado =  eval( "UsuarioX" + params[:objeto].split('_').map(&:capitalize).join + ".create(:usuario_id => params[:usuario][:usuario_id], :" + params[:objeto] + "_id =>  params[:id], :rol =>  params[:usuario][:rol])" ) if params[:usuario]
      params[:usuario_id] = params[:usuario][:usuario_id]
    end
    @usuarios_x = @objeto.usuario_x_vinculado

    if @usuario_relacionado.errors.empty?
      usuario_asignado = Usuario.find_by_id( params[:usuario_id] )
      # Si queremos propagar permisos (en (, lo hacemos
      if params[:selector] && params[:selector][:forzar_permisos] && params[:selector][:forzar_permisos] == "1"
        # Asigna los implementadores
        @objeto.implementador.each do |implementador|
          UsuarioXAgente.find_or_create_by_usuario_id_and_agente_id(usuario_asignado.id,implementador.id)
        end if usuario_asignado && @objeto.class.name == "Proyecto"
        # Asigna los libros
        @objeto.libro.each do |libro|
          UsuarioXLibro.find_or_create_by_usuario_id_and_libro_id(usuario_asignado.id,libro.id)
        end if usuario_asignado
      end
      # Envia correo de advertencia
      for usuario in @objeto.usuario.uniq
        begin
          Correo.asignar_usuario(request.host_with_port, usuario, usuario_asignado, params[:usuario] ? params[:usuario][:rol] : nil, @objeto).deliver
        rescue
          @message = _("No se ha podido mandar el mail a algún usuario")
        end
      end
    end
    render :update do |page|
      page.replace params[:update] + "_usuarios", :partial => "usuario/usuarios"
      page.replace 'formulario', :inline => '<%= mensaje_error(@usuario_relacionado ? (@message || @usuario_relacionado) : "Se han producido errores. No se guardan los datos.") %><br>'
    end 
  end

  def desasignar_usuario
    @objeto = eval( params[:objeto].split('_').map(&:capitalize).join ).find( params[:id].to_s )
    eval( "UsuarioX" + params[:objeto].split('_').map(&:capitalize).join + ".find( params[:usuario_x_objeto_id] ).destroy" )
    @usuarios_x = @objeto.usuario_x_vinculado
    render :update do |page|
      page.replace params[:update] + "_usuarios", :partial => "usuario/usuarios"
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@objeto, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
    end
  end


  #++
  # Registro y aviso legal
  #--

  # Envio del formulario de registro
  def registro_envio
    if ENV['DEMO'] && simple_captcha_valid?
      if params[:registro] &&
         params[:registro][:nombre] != "" && params[:registro][:correoe] != "" &&
         params[:registro][:pais] != "" && params[:registro][:empresa] != "" && 
         params[:registro][:interes] != ""
        begin
          # Situación provisional. Habria que pensar en la necesidad de crear usuarios, vincularlo a proyectos,...
          destinatario = params[:registro][:nombre] + " <" + params[:registro][:correoe] + ">"
          Correo.registro(request.host_with_port, destinatario, params[:registro], "admin", DEMO_PASS).deliver
          logger.info "----------------> Se ha enviado correo de registro" + destinatario
          Correo.registro(request.host_with_port, DEMO_CORREO_INFO, params[:registro], "admin", DEMO_PASS).deliver
          logger.info "----------------> Se ha enviado correo de registro" + DEMO_CORREO_INFO
          msg_error _("Gracias por registrarse. Revise su correo con el usuario y contraseña necesarios para acceder al sistema"), :ok => true
        rescue => ex
          logger.error "----------------> Se ha producido un error enviando algun mail: " + ex.inspect
          msg_error _("Lo sentimos. No se ha podido enviar el correo con la información para el registro")
        end
      else
        msg_error _("Rellene todos los datos del formulario")
      end
    else
      msg_error _("El código introducido no coincide con la imagen") if ENV['DEMO']
    end
    redirect_to :entrada and return
  end

  # Aviso legal
  def avisolegal
    if request.xhr?
      render :update do |page|
        page.replace_html "explicacion_cuerpo", :partial => "avisolegal_" + (FastGettext.locale||"es")[0..1]
      end
    else
      redirect_to :proyectos
    end
  end

 private

  # Devuelve el usuario enviado por parametros
  def usuario_desde_parametros
    # Si estamos en una delegacion, averiguamos el rol del agente en ella
    rol_agente = @agente ? @usuario_identificado.usuario_x_agente.where(agente_id: @agente.id) : [] 
    # Solo devolvemos si el usuario que ejecuta es administrador o
    if @usuario_identificado.administracion || ( rol_agente.include?("coordinador") ) || ( rol_agente.include?("economico") )
      @usuario = Usuario.find_by_id params[:id]
    end
  end

  # Devuelve los usuario_x_proyecto para un usuario concreto
  def usuario_x_proyecto
    @usuario.usuario_x_proyecto.joins(:proyecto).where("proyecto.convenio_id" => nil).joins(:proyecto => [:estado_actual => :definicion_estado]).where("definicion_estado.cerrado" => false).order("proyecto.nombre")
  end

  def usuario_x_agente
    @usuario.usuario_x_agente.joins(:agente).where("agente.implementador" => true, "agente.socia_local" => false).order("agente.nombre")
  end

  def usuario_x_libro
    @usuario.usuario_x_libro.joins(:libro).where("libro.bloqueado" => false).order("libro.nombre")
  end

  def elementos_posibles
    @elementos = []
    case params[:tipo]
      when "proyecto"
        @elementos = Proyecto.where(convenio_id: nil).joins(estado_actual: :definicion_estado).where("definicion_estado.cerrado" => false).order("nombre") - @usuario.proyecto.where("usuario_x_proyecto.grupo_usuario_id" => nil)
      when "agente"
        @elementos = Agente.where(implementador: true, socia_local: false).order("nombre") - @usuario.agente.where("usuario_x_agente.grupo_usuario_id" => nil)
      when "libro"
        @elementos = Libro.where(bloqueado: false).order("nombre") - @usuario.libro.where("usuario_x_libro.grupo_usuario_id" => nil)
    end
    @elementos.push @elemento.send(params[:tipo]) if @elemento && @elemento.id
  end
 
end
