# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++

class Correo < ActionMailer::Base
  if self.included_modules.include?(AbstractController::Callbacks)
    raise "You've already included AbstractController::Callbacks, remove this line."
  else
    include AbstractController::Callbacks
  end

  before_filter :add_inline_attachments!

  # FROM = "gong.development@gmail.com"
  default :from => 'no-reply@' + (ENV['GOR_SITEID'] || 'gong.org.es') 

  # es llamado por EstadoController::cambio_estado para generar el mail de notificación
  def cambio_estado(host, usuario, tipo, proyecto, estado)
    @host = host
    @usuario = usuario
    @proyecto = proyecto
    @estado = estado
    variables_comentario("", host, nil, proyecto)
    mail :to => "#{usuario.nombre_completo} <#{usuario.correoe}>", :subject => "[GONG] " + proyecto.nombre + ": " + _("Cambio de estado") 
  end

  def cambio_tarea(host, usuario, tarea)
    @usuario = usuario
    @tarea = tarea
    objeto = @tarea.proyecto || @tarea.agente
    texto_subject  = "[GONG] "
    texto_subject += (objeto.nombre + ": ") if objeto
    texto_subject += _("Noficación de Tarea")
    variables_comentario("Tarea", host, tarea, objeto)
    mail :to => "#{usuario.nombre_completo} <#{usuario.correoe}>", :subject => texto_subject 
  end

  def nuevo_comentario(host, usuario, tipo, objeto, objeto_relacionado, comentario)
    @usuario = usuario
    @objeto = objeto 
    @objeto_relacionado = objeto_relacionado
    @comentario = comentario 
    # Solo notifica si el objeto no es un proyecto, o siendolo permitimos notificaciones suyas
    if ( objeto.class.name != "Proyecto" || (@usuario.usuario_x_proyecto.find_by_proyecto_id(objeto.id) && @usuario.usuario_x_proyecto.find_by_proyecto_id(objeto.id).notificar) )
      variables_comentario(tipo, host, objeto, objeto_relacionado)
      mail :to => "#{usuario.nombre_completo} <#{usuario.correoe}>", :subject => "[GONG] " + (objeto ? objeto.nombre + ": " : "") + _("Nuevo Comentario en") + " " + _(objeto_relacionado.class.name)
    end
  end

  def asignar_usuario(host, usuario, usuario_asignado, rol, objeto)
    @usuario = usuario
    @usuario_asignado = usuario_asignado
    @objeto = objeto
    @objeto_nombre = objeto.nombre unless objeto.class.name == "Espacio"
    @objeto_nombre = objeto.ruta + " / " + objeto.nombre if objeto.class.name == "Espacio"
    variables_comentario("", host, usuario, objeto)
    subject = _("Usuario asignado al") + " " + _(objeto.class.name.capitalize) + ": " + objeto.nombre 
    mail :to => "#{usuario.nombre_completo} <#{usuario.correoe}>", :subject => "[GONG] " + objeto.nombre + ": " + _("Usuario asignado") 
  end

  def registro (host, destinatario, registro, nombre_usuario, contraseña)
    @registro, @nombre_usuario, @contrasena = registro, nombre_usuario, contraseña 
    @url = host
    mail :to => destinatario, :subject => "[GONG] " + _("Bienvenido a la demo de GONG")
  end



 private
  def variables_comentario tipo, host, objeto, objeto_relacionado
    url = {:seccion => tipo, :host => host, :only_path => false}
    if tipo == "proyectos"
      estado_proyecto = (objeto.estado_actual.definicion_estado.formulacion ? "formulacion" : "ejecucion_tecnica") if objeto.estado_actual
      url[:proyecto_id] = objeto.id
      case objeto_relacionado.class.name
        when "Presupuesto"
          url[:menu] = "formulacion"
          url[:controller] = "presupuesto_proyectos"
          descripcion = objeto_relacionado.concepto
        when "Gasto"
          url[:menu] = "ejecucion_economica"
          url[:controller] = "gasto_proyectos"
          descripcion = "#{I18n.l objeto_relacionado.fecha} (#{objeto_relacionado.importe} #{objeto_relacionado.moneda.abreviatura}): #{objeto_relacionado.concepto}"
        when "Tarea"
          url[:menu] = "resumen"
          url[:controller] = "tarea"
          descripcion = objeto_relacionado.titulo
        when "Documento"
          url[:menu] = "documentos_proyecto"
          url[:controller] = "documento"
          descripcion = objeto_relacionado.adjunto_file_name
        when "Actividad"
          url[:menu] = estado_proyecto
          url[:controller] = "actividad"
          descripcion = objeto_relacionado.codigo
        when "Subactividad"
          url[:menu] = estado_proyecto
          url[:controller] = "actividad"
          descripcion = objeto_relacionado.descripcion
        when "Indicador"
          url[:menu] = estado_proyecto
          url[:controller] = "indicador"
          descripcion = objeto_relacionado.codigo
        when "FuenteVerificacion"
          url[:menu] = estado_proyecto
          url[:controller] = "fuente_verificacion"
          descripcion = objeto_relacionado.codigo
        when "ObjetivoEspecifico"
          url[:menu] = estado_proyecto
          url[:controller] = "matriz"
          descripcion = objeto_relacionado.codigo + " " + objeto_relacionado.descripcion
        when "Resultado"
          url[:menu] = estado_proyecto
          url[:controller] = "matriz"
          descripcion = objeto_relacionado.codigo + " " + objeto_relacionado.descripcion
      end
    elsif tipo == "agentes"
      url[:agente_id] = objeto.id
      case objeto_relacionado.class.name
        when "Presupuesto"
          url[:menu] = "economico_agente"
          url[:controller] = "presupuesto_agentes"
          descripcion = objeto_relacionado.concepto + (objeto_relacionado.etapa ? " (" + objeto_relacionado.etapa.nombre + ")" : "")
        when "Gasto"
          url[:menu] = "economico_agente"
          url[:controller] = "gasto_agentes"
          descripcion = "#{I18n.l objeto_relacionado.fecha} (#{objeto_relacionado.importe} #{objeto_relacionado.moneda.abreviatura}): #{objeto_relacionado.concepto}"
        when "Tarea"
          url[:menu] = "resumen_agente"
          url[:controller] = "tarea"
          descripcion = objeto_relacionado.titulo
        when "Documento"
          url[:menu] = "documentos_agente"
          url[:controller] = "documento"
          descripcion = objeto_relacionado.adjunto_file_name
        else
          url[:menu] = "resumen_agente"
          url[:controller] = "info"
          descripcion = objeto.nombre
      end
    elsif tipo == "Tarea"
      case objeto_relacionado.class.name
        when "Proyecto"
          url[:seccion] = "proyectos"
          url[:menu] = "resumen"
          url[:controller] = "tarea"
          url[:proyecto_id] = objeto_relacionado.id
          descripcion = objeto_relacionado.nombre
        when "Agente"
          url[:seccion] = "agentes"
          url[:menu] = "resumen_agente"
          url[:controller] = :tarea
          url[:agente_id] = objeto_relacionado.id
          descripcion = objeto_relacionado.nombre
        else
          url[:seccion] = "inicio"
          url[:controller] = :tarea
          descripcion = "Gong" 
      end
    else
      case objeto_relacionado.class.name
        when "Proyecto"
          url[:seccion] = "proyectos"
          url[:menu] = "resumen"
          url[:controller] = :info
          url[:proyecto_id] = objeto_relacionado.id
          descripcion = objeto_relacionado.nombre
        when "Agente"
          url[:seccion] = "agentes"
          url[:menu] = "resumen_agente"
          url[:controller] = :tarea
          url[:agente_id] = objeto_relacionado.id
          descripcion = objeto_relacionado.nombre
        when "Tarea"
          url[:seccion] = "inicio"
          url[:controller] = :tarea
          descripcion = objeto_relacionado.titulo
        else
          url[:seccion] = "inicio"
          url[:controller] = "info"
          descripcion = "Gong"
      end
    end
    @url = url
    @texto_url = descripcion
  end

 private
  def add_inline_attachments!
    attachments.inline['logo_gong_boletin.jpg'] = File.read('app/assets/images/logo_gong_boletin.jpg')
    attachments.inline['cabecera_boletin.jpg'] = File.read('app/assets/images/cabecera_boletin.jpg')
    attachments.inline['logo_gong_pie.jpg'] = File.read('app/assets/images/logo_gong_pie.jpg')
    attachments.inline['licencia_creativecommons.jpg'] = File.read('app/assets/images/licencia_creativecommons.jpg')
  end
end
